/**
 *                                                                                                                                                                                                                                                              Copyright (c) 2016-2019 人人开源 All rights reserved.
 *
 * https://www.renren.io
 *
 * 版权所有，侵权必究！
 */

package com.xyh.mychatgpt.utils;

import cn.hutool.json.JSONUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.*;
import org.springframework.stereotype.Component;
import java.util.concurrent.TimeUnit;

/**
 * Redis工具类
 */
@Component
public class RedisUtils {
    @Autowired
    private RedisTemplate<String, Object> redisTemplate;
    @Autowired
    private ValueOperations<String, String> valueOperations;
    @Autowired
    private ListOperations<String, Object> listOperations;

    /**  默认过期时长，单位：秒 */
    public final static long DEFAULT_EXPIRE = 60 * 60 * 24;
    /**  不设置过期时长 */
    public final static long NOT_EXPIRE = -1;

    public void set(String key, Object value, long expire){
        valueOperations.set(key, toJson(value));
        if(expire != NOT_EXPIRE){
            redisTemplate.expire(key, expire, TimeUnit.SECONDS);
        }
    }


    public void set(String key, Object value){
        set(key, value, DEFAULT_EXPIRE);
    }

    public <T> T get(String key, Class<T> clazz, long expire) {
        String value = valueOperations.get(key);
        if(expire != NOT_EXPIRE){
            redisTemplate.expire(key, expire, TimeUnit.SECONDS);
        }
        return value == null ? null : fromJson(value, clazz);
    }

    public <T> T get(String key, Class<T> clazz) {
        return get(key, clazz, NOT_EXPIRE);
    }

    public String get(String key, long expire) {
        String value = valueOperations.get(key);
        if(expire != NOT_EXPIRE){
            redisTemplate.expire(key, expire, TimeUnit.SECONDS);
        }
        return value;
    }

    public String get(String key) {
        return get(key, NOT_EXPIRE);
    }

    public void delete(String key) {
        redisTemplate.delete(key);
    }

    /**
     * 从key列表左边（从头部）插入 值
     * @param key 不能为空
     * @param value 插入列表的值
     * @return 返回列表插入后的长度
     */
    public Long leftPush(String key,Object value){
      return listOperations.leftPush(key,value);
    }

    /**
     * 移除列表中的第一个值，并返回该值
     * @param key 不能为空
     * @return 移除的值
     **/
    public Object leftPop(String key){
       return listOperations.leftPop(key);
    }

    /**
     * 从key列表右侧插入值
     * @param key 不能为空
     * @param value 插入列表的值
     * @return 返回列表插入后的长度
     */
    public Long rightPush(String key,Object value){
        return listOperations.rightPush(key, value);
    }
    /**
     * 移除列表中的最后一个值，并返回该值
     * @param key 不能为空
     * @return 移除的值
     **/
    public Object rightPop(String key){
        return listOperations.rightPop(key);
    }

    /**
     * 获取key 列表的长度
     *
     * @param key 不能为null
     * @return 列表长度
     */
    public Long listSize(String key){
        return listOperations.size(key);
    }

    /**
     * Object转成JSON数据
     */
    private String toJson(Object object){
        if(object instanceof Integer || object instanceof Long || object instanceof Float ||
                object instanceof Double || object instanceof Boolean || object instanceof String){
            return String.valueOf(object);
        }
        return JSONUtil.toJsonStr(object);
    }

    /**
     * JSON数据，转成Object
     */
    private <T> T fromJson(String json, Class<T> clazz){

        return JSONUtil.toBean(json,clazz);
    }
}
